-- copyright benjamin bober, 2020
qualityoflife = qualityoflife or {};
local q = qualityoflife;

-- string functions

function q:Println(str, moduleName)
	if str then
		if moduleName then
			print("[QoL] [" .. moduleName .. "] " .. str);
		else
			print("[QoL] " .. str);
		end
	end
end

function q:StartsWith(str, substr, caseInsensitive)
	if caseInsensitive then
		str = string.upper(str);
		substr = string.upper(substr);
	end

   return string.sub(str, 1, string.len(substr)) == substr;
end

function q:StringContains(str, match, caseInsensitive)
	if caseInsensitive then
		str = string.upper(str);
		match = string.upper(match);
	end

	return string.find(str, "%" .. match);
end

function q:Trim(str)
	if str then
		return str:gsub("^%s*(.-)%s*$", "%1");
	end
	return "";
end

function q:BoolToText(value, trueText, falseText)
	trueText = trueText or q:GetText("yes");
	falseText = falseText or q:GetText("no");

	if value then
		return trueText;
	end
	return falseText;
end

function q:StringHash(text)
	if text then
		local counter = 1
		local len = string.len(text)
		for i = 1, len, 3 do
		counter = math.fmod(counter*8161, 4294967279) +  -- 2^32 - 17: Prime!
			(string.byte(text,i)*16776193) +
			((string.byte(text,i+1) or (len-i+256))*8372226) +
			((string.byte(text,i+2) or (len-i+256))*3932164)
		end
		return math.fmod(counter, 4294967291) -- 2^32 - 5: Prime (and different from the prime in the loop)
	end
end

-- table functions

function q:GetTableSize(t)
    local count = 0
    for _, __ in pairs(t) do
        count = count + 1
    end
    return count
end

function q:GetSortedTableKeys(t, descending)
	if t then
		local keys = {};
		for k, _ in pairs(t) do
			local num = tonumber(k);
			if num then
				table.insert(keys, num);
			else
				table.insert(keys, k);
			end
		end
		if descending then
			table.sort(keys, function(a, b) return a > b end); -- descending
		else
			table.sort(keys);
		end
		return keys;
	end
end

function q:DumpTable(t, key)
	-- primitive debug function
	if t then
		if type(t) == "table" then
			for k, v in pairs(t) do
				q:DumpTable(v, k);
			end
		elseif key then
			print(key .. "->", t);
		else
			print(t);
		end
	end
end

function q:TableIsArray(t)
	return t[1] and q:GetTableSize(t) == table.getn(t);
end

function q:TableContains(t, value, caseInsensitive)
	if caseInsensitive then
		value = string.upper(value);
	end

	if q:TableIsArray(t) then
		for k, v in ipairs(t) do
			if (not caseInsensitive and v == value) or (caseInsensitive and string.upper(v) == value) then
				return v;
			end
		end
	else
		for k, v in pairs(t) do
			if (not caseInsensitive and v == value) or (caseInsensitive and string.upper(v) == value) then
				return v;
			end
		end
	end
end

-- other/general functions

function q:Deepcopy(orig, copies)
    copies = copies or {};
    local orig_type, copy = type(orig);
    if orig_type == "table" then
        if copies[orig] then
            copy = copies[orig];
        else
            copy = {};
            copies[orig] = copy;
            setmetatable(copy, q:Deepcopy(getmetatable(orig), copies));
            for orig_key, orig_value in next, orig, nil do
                copy[q:Deepcopy(orig_key, copies)] = q:Deepcopy(orig_value, copies);
            end
        end
    else
        copy = orig;
    end
    return copy;
end

function q:GetColorRBGA(colorTable)
	if colorTable then
		return colorTable["r"], colorTable["g"], colorTable["b"], colorTable["a"];
	end
end

function q:GetColorHexFromRGBA(colorTable, g, b, a)
	local r;
	if g and b and a then
		r = colorTable;
	else
		r, g, b = q:GetColorRBGA(colorTable);
	end

	-- alpha is ignored anyway by blizzard "|c"
	return "ff" .. string.format("%02x", r * 255) .. string.format("%02x", g * 255) .. string.format("%02x", b * 255);
end

function q:HandleElvUi(frame, additionalActions)
	if ElvUI and frame and frame.frameType then
		local E, L, V, P, G = unpack(ElvUI);
		local elv = E:GetModule('Skins');

		if frame.frameType == "Frame" then
			elv:HandleFrame(frame);
		elseif frame.frameType == "DropDown" then
			elv:HandleDropDownBox(frame);
		elseif frame.frameType == "Button" then
			elv:HandleButton(frame);
		elseif frame.frameType == "CheckButton" then
			elv:HandleCheckBox(frame);
		elseif frame.frameType == "Slider" then
			elv:HandleSliderFrame(frame);
		elseif frame.frameType == "EditBox" then
			elv:HandleEditBox(frame);
		else
			return;
		end

		if additionalActions then
			additionalActions(frame);
		end
	end
end